<?php

namespace Drupal\uaparser;

use Drupal\Component\Utility\Timer;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\State\StateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use UAParser\Parser as UAParser;
use UAParser\Util\Converter;
use UAParser\Util\Fetcher;
use Psr\Log\LoggerInterface;

/**
 * A service class to integrate with ua-parser.
 */
class Parser implements ParserInterface {

  use StringTranslationTrait;

  /**
   * The cache service.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cache;

  /**
   * The state service.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * The ua-parser logger.
   *
   * @var \Psr\Log\LoggerInterface.
   */
  protected $logger;

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a Parser object.
   *
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache_service
   *   The cache service.
   * @param \Drupal\Core\State\StateInterface $state
   *   The state service.
   * @param \Psr\Log\LoggerInterface $logger
   *   The ua-parser logger.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   */
  public function __construct(CacheBackendInterface $cache_service, StateInterface $state, LoggerInterface $logger, ConfigFactoryInterface $config_factory) {
    $this->cache = $cache_service;
    $this->state = $state;
    $this->logger = $logger;
    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public function update($set_message = FALSE) {
    try {
      $fetcher = new Fetcher();
      $converter = new Converter($this->configFactory->get('uaparser.settings')->get('regexes_file_location'));
      $converter->convertString($fetcher->fetch());
      $this->state->set('uaparser.last_update', REQUEST_TIME);
      $this->cache->deleteAll();
      $this->logger->notice('ua-parser user-agent definitions file update executed successfully.');
    }
    catch (\Exception $e) {
      $this->logger->error('ua-parser user-agent definitions file update failed, error: @error', ['@error' => $e->getMessage()]);
      if ($set_message) {
        drupal_set_message($this->t('Update error: @error', ['@error' => $e->getMessage()]), 'error');
      }
      return FALSE;
    }
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function parse($ua, $use_cache = TRUE) {
    if ($use_cache && ($cache = $this->cache->get($ua))) {
      $result = $cache->data;
    }
    else {
      $result = [];
      try {
        $file = $this->configFactory->get('uaparser.settings')->get('regexes_file_location') . DIRECTORY_SEPARATOR . 'regexes.php';
        if (!file_exists($file)) {
          $this->update();
        }
        Timer::start('uaparser:parse');
        $parser = UAParser::create($file);
        $result['client'] = $parser->parse($ua);
        $time = Timer::stop('uaparser:parse')['time'];
        if ($use_cache) {
          $this->cache->set($ua, $result, Cache::PERMANENT, []);
        }
        $result['time'] = $time;
      }
      catch (\Exception $e) {
        $result['error'] = $e->getMessage();
      }
    }

    return $result;
  }

}
